#!/usr/bin/perl -w

# Copyright 2010 Kevin Ryde

# This file is part of Math-PlanePath.
#
# Math-PlanePath is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# Math-PlanePath is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Math-PlanePath.  If not, see <http://www.gnu.org/licenses/>.


  #     if (($self->{'show'}||'') eq 'dump-list') {
  #       foreach my $i (1 .. $n_pixels-1) {
  #         my ($x, $y) = $path->n_to_xy ($i);
  #         if (! defined $x) { last; }
  #         $x += $x_origin;
  #         $y += $y_origin;
  #         printf "%d  %s,%s\n", $i, $x//'undef', $y//'undef';
  #       }
  #       exit 0;
  #     }
  #

